/* Mini Reel for cut tape with SMD components, version 3

   This is a smaller version of the standard 7-inch reel.
   See EIA-481 for design dimensions and standardized dimensions.

   Copyright 2021-2023, CompuPhase, Thiadmer Riemersma
   This work is licensed under a Creative Commons Attribution-ShareAlike 4.0 International License
   (https://creativecommons.org/licenses/by-sa/4.0/).
  */

/* [Options] */

// The width of the tape (note that some tolerance is added to this).
Tape_Width = 8;       // 1

// Whether to include a slot for estimating how much is TopSide on the reel.
Scale_Slot = true;

// The offset that the scale slot backs off from the edge. Recommended value is 2mm.
Scale_Slot_Offset = 2;

// Whether to add a pin in the slots (where the tape is inserted) that snap into the sprocket holes the tape, in order to avoid the tape to slide out too easily. If set to true, these slots are also made wider.
Sprocket_Pins = true;

// Whether to add a TAB for a label on one flange (size for the tab is in the "Design dimensions")
Label_Tab = false;

/* [Design dimensions] */

// The diameter of the mini-reel; common commercial mini-reels were 100mm.
Diameter = 100;

// The diameter of the inner cylinder that the tape is wound around; 50mm is generally regarded as the mimimum value.
Hub = 50;

// The thickness of the wall for the hub on the top side (recommended is at least 2 perimeters, but more than 4 perimeters is overkill).
Hub_Wall_Width = 1.7;

// The hub must be slightly wider than the tape; the recommended tolerance is 0.2mm to 0.5mm.
Tape_Hub_Tolerance = 0.4;

// The thickness of the flanges (the disks or sides of the reel); 0.8mm to 1.2mm is recommended in the standards.
Flange_Thickness = 1.0; // 0.1

// The width of the spokes of the flanges, between the hub and the outer edge.
Flange_Spokes_Width = 25;

// The width of the outer ring of the flanges (the disks or sides of the reel); a larger width makes the openings in the flanges narrower.
Flange_Edge_Width = 8;

// It may be useful for the tape to be clamped between the flanges at the edges, so that the tape doesn't easily unroll. Recommended values are between 0.2mm and 0.6mm.
Flange_Edge_Rim_Height = 0.2;

// The width of the rim, recommended value is between 1mm and 1.5mm.
Flange_Edge_Rim_Width = 1.2;

// Height (or "width") of the label TAB (see "Options" for whether a TAB is added).
Label_Tab_Height = 6;

// Length of the label TAB (see "Options" for whether a TAB is added).
Label_Tab_Length = 25;

// Diameter of the hole in the label; set to zero for no hole.
Label_Tab_Hole = 3; // 0.1

// For fitting the two parts together, some tolerance may be needed.
Tolerance = 0.1;      /* for fitting the two sides together */

/* [Standardized dimenions] */

// The diameter of the spindle should be in the range 12.8mm to 13.5mm (the spindle is also called "arbor").
Spindle_Diameter = 13;  // 0.5

// The width of the slots for the three "wings" (or "spokes") on the spindle; standardized minimum value is 1.5mm, but 2mm is recommended.
Spindle_Wing_Width = 2; // 0.1

// The length of the slots for the three "wings" (or "spokes") on the spindle; standardized minimum length (from the centre) is 10.1mm.
Spindle_Wing_Length = 10.3;

// The sprocket holes in the edge of the tape are separated by 4mm.
Sprocket_Hole_Pitch = 4.0;  // 0.1

// The diameter of the sprocket holes in tape, standardized at 1.5mm (minimum).
Sprocket_Hole_Size = 1.5;

// The offset of the sprocket holes from the edge of the tape.
Sprocket_Hole_Edge_Offset = 1.75;

/* [Special] */

// Whether three slots for tape ends should be added at all.
Tape_End_Slots = true;

// Diameter of the three alignment pins (to fit top to bottom).
Alignment_Pin_Diameter = 4;

// Whether a full cover (that slips over the reel) is included.
Include_Cover = false;

/* [Hidden] */

TopSide = 1; // these two variables are used as symbolic constants
BottomSide = 2;

epsilon = 0.05;
$fn = 100;


module spindle(height) {
  /* positive image of the spindle, should normally be used in a difference() block */
  translate([0, 0, -epsilon]) {
    union() {
      cylinder(d=Spindle_Diameter, h=height + 2*epsilon);
      for (idx = [0 : 2]) {
        rotate([0, 0, 120 * idx + 60])
          translate([0, -Spindle_Wing_Width/2, 0])
            cube([Spindle_Wing_Length, Spindle_Wing_Width, height + 2*epsilon]);
      }
    }
  }
}

wd_mult = Tape_Width / 8;
wd_tolerance = 0.1;     /* slots may come out narrower than calculated, so slightly adjust width */
sprocketpin = Sprocket_Pins ? (Sprocket_Hole_Size - 0.1) : 0; /* sprocket holes in tape are defined at 1.5 mm, so the pin must be slictly smaller */
function slot_width(side, index) = ((side == BottomSide) ? index : (4 - index)) * wd_mult + wd_tolerance + sprocketpin*0.35;

module tape_slots(side) {
  /* positive image of tape slots, should be used in a difference() block */
  union() {
    ringheight = Tape_Width + Tape_Hub_Tolerance + Flange_Thickness;
    offs = Spindle_Diameter/2 + 5;  /* +5 is start of the slot from the edge of the spindle */
    for (idx = [1 : 3]) {
      wd = Tape_End_Slots ? slot_width(side, idx) : 0.5;
      rotate([0, 0, 120 * (idx - 1)])
        translate([offs, -wd/2, -epsilon])
          cube([Hub/2 - offs, wd, ringheight + 2*epsilon]);
    }
  }
}

module sprocket_pin(slot_index) {
  /* a cylinder with a rounded end */
  extra_length = min(slot_index*wd_mult*0.15, 0.5);
  union() {
    if (extra_length > 0)
      translate([extra_length, 0, 0])
        rotate([0, -90, 0])
          cylinder(d = sprocketpin, h = extra_length+epsilon);
    translate([extra_length, 0, 0])
      sphere(d = sprocketpin);
  }
}

module sprocket_pins() {
  zpos = Flange_Thickness + Tape_Hub_Tolerance/2 + Sprocket_Hole_Edge_Offset;
  xpos = Hub/2 - Hub_Wall_Width - Sprocket_Hole_Pitch;
  union() {
    for (idx = [1 : 3]) {
      rotate([0, 0, 120 * idx + 150]) {
        wd = - slot_width(BottomSide, idx);
        translate([wd/2, xpos, zpos])
          sprocket_pin(idx);
        translate([wd/2, xpos-Sprocket_Hole_Pitch, zpos])
          sprocket_pin(idx);
      }
    }
  }
}

module alignment_pins(side) {
  /* used "positive" for pins and "negative" for pin holes; holes are in bottom side */
  pin_height = 2;   /* base depth, excluding tolerance */
  extra_hole_depth = 0.5; /* for tolerance (it does not hurt for the hole being too deep) */
  union() {
    offs = Spindle_Wing_Length / 2 + (Hub - 2*Hub_Wall_Width) / 4;
    diam = (side == BottomSide) ? Alignment_Pin_Diameter + Tolerance : Alignment_Pin_Diameter;
    height = (side == BottomSide) ? pin_height + extra_hole_depth : pin_height;
    z_offs = (side == BottomSide) ? Tape_Width + Tape_Hub_Tolerance + Flange_Thickness - height + epsilon : Flange_Thickness;
    for (idx = [1 : 3]) {
      rotate([0, 0, 120 * (idx - 1) + 60])
        translate([offs, 0, z_offs])
          cylinder(d = diam, h = height);
    }
  }
}

module groove_cylinder(diameter, height, groove_height, groove_width) {
  union() {
    cylinder(d = diameter - 2*groove_width, h = height);
    translate([0, 0, groove_height])
      cylinder(d = diameter, h = height - 2*groove_height);
  }
}

module hub(side) {
  ringheight = Tape_Width + Tape_Hub_Tolerance;
  groove_width = 0.25; /* groove for snaplock (of the outer hub on the inner hub) */
  groove_height = 0.5;
  union() {
    difference() {
      if (side == TopSide) {
        /* big (hollow) ring */
        difference() {
          translate([0, 0, Flange_Thickness - epsilon])
            cylinder(d=Hub, h=ringheight + epsilon);
          translate([0, 0, Flange_Thickness - epsilon])
            groove_cylinder(Hub - 2*Hub_Wall_Width, ringheight + 2*epsilon, groove_height, groove_width);
        }
      } else {
        /* smaller (solid) ring */
        translate([0, 0, Flange_Thickness - epsilon])
          groove_cylinder(Hub - 2*Hub_Wall_Width - Tolerance, ringheight - Tolerance, groove_height + Tolerance, groove_width);
      }

      /* slots for the tape (even if no tape slots are desired, narrow slots
         in the top flange are required to be able to fit the two halves */
      if (Tape_End_Slots || side == TopSide)
        tape_slots(side);
    }
    if (Tape_End_Slots && Sprocket_Pins && side == BottomSide)
      sprocket_pins();
  }
}

module pie_shape_rounded(inner_radius, outer_radius, angle, spokeoffset, rounding, height) {
  /* 3D "pie slice" shape with inner & outer radius, and rounded corners */
  linear_extrude(height) {
    minkowski() {
      assert(angle <= 180); /* this method does not work for slices of over a half circle */
      difference() {
        circle(outer_radius - rounding);
        circle(inner_radius + rounding);
        union() {
          cut_size = 2 * ((outer_radius - rounding) + epsilon);
          translate([-cut_size/2, -(spokeoffset + rounding)]) 
            square(cut_size);
          rotate(180 - angle) 
            translate([-cut_size/2, -(spokeoffset + rounding)]) 
              square(cut_size);
        }
      }
      circle(r = rounding);
    }
  }
}

module label_tab() {
  radius = 3;
  tab_thickness = Flange_Thickness + Flange_Edge_Rim_Height;
  /* calculate extra extent of the label, to fit on the circular curve of the flange */
  //alpha = asin((Label_Tab_Length + 2*radius + epsilon) / Diameter);
  alpha = asin((Label_Tab_Length + 2*Label_Tab_Height + epsilon) / Diameter);
  negative_extent = (Diameter / 2) * (1 - cos(alpha));
  difference() {
    union() {
      /* main TAB shape */
      hull() {
        translate([-(Label_Tab_Length/2 - radius), Label_Tab_Height - radius, 0])
          cylinder(r = radius, h = tab_thickness);
        translate([Label_Tab_Length/2 - radius, Label_Tab_Height - radius, 0])
          cylinder(r = radius, h = tab_thickness);
        translate([-(Label_Tab_Length/2 + Label_Tab_Height), -negative_extent, 0])
          cube([Label_Tab_Length + 2*Label_Tab_Height, epsilon, tab_thickness]);
      }
    }
    /* cut off the cylinder of the flange */
    translate([0, -Diameter/2, -epsilon])
      cylinder(d = Diameter - epsilon, h = tab_thickness + 2*epsilon);
    /* add the hole (optionally) */
    if (Label_Tab_Hole > 0.2) {
      offs_y = Label_Tab_Height - Label_Tab_Hole/2 - max(Flange_Edge_Rim_Width, 1);
      translate([0, offs_y, -epsilon])
        cylinder(d = Label_Tab_Hole, h = tab_thickness + 2*epsilon);
    }
  }
}

module disk(side) {
  difference() {
    /* disk with rim */
    cylinder(d=Diameter, h=Flange_Thickness + Flange_Edge_Rim_Height);
    if (Flange_Edge_Rim_Height > epsilon && Flange_Edge_Rim_Width > epsilon) {
      translate([0, 0, Flange_Thickness])
        cylinder(d=Diameter - 2*Flange_Edge_Rim_Width, h=Flange_Edge_Rim_Height + epsilon);
    }

    /* pie-shaped slotes, for putting the tape in the slot */
    num_holes = (side == BottomSide) ? 3 : 2;
    slice_rounding = (Diameter - Flange_Edge_Width - Hub) / 10;
    for (idx = [1 : num_holes]) {
      rotate([0, 0, 120 * (idx + 1) + 60])
        translate([0, 0, -epsilon])
          pie_shape_rounded(Hub/2, Diameter/2 - Flange_Edge_Width, 120, Flange_Spokes_Width/2, slice_rounding, Flange_Thickness + 2*epsilon);
    }

    /* slots for the tape (only in the bottom flange) */
    if (side == BottomSide && Tape_End_Slots)
      tape_slots(side);
  }
}

module reel_half(side) {
  fullheight = Tape_Width + Tape_Hub_Tolerance + 2*Flange_Thickness;
  difference() {
    union() {
      hub(side);
      disk(side);
      if (side == TopSide) {
        alignment_pins(side); /* pins */
      } else if (Label_Tab) {
        rotate([0, 0, 60])
          translate([0, Diameter/2, 0])
            label_tab();
      }
    }
    /* hole for the spindle goes through */
    spindle(fullheight);
    if (side == BottomSide)
      alignment_pins(side); /* holes */

    /* slot for estimating the amount left */
    if (Scale_Slot && side == TopSide) {
      assert(Scale_Slot_Offset > Flange_Edge_Rim_Width);  /* slot must not cut into the rim */
      scale_slot_width = 2;
      slot_length = (Diameter - Hub) / 2 - Scale_Slot_Offset;
      translate([-Diameter/2 + Scale_Slot_Offset, -scale_slot_width/2, -epsilon])
        cube([slot_length + Hub_Wall_Width + epsilon, scale_slot_width, Flange_Thickness + 2*Tape_Hub_Tolerance]);

      /* calculate the "quantity" for a full reel (the value is actually the
         length of the tape multiplied by the tape thickness and divided pi) */
      Cfull = (slot_length + Hub) * slot_length;
      scalemark_length = 2.5; /* for the short mark, long mark is twice this value */
      scalemark_width = 0.1;
      scalemark_depth = min(Flange_Thickness / 2, 0.4);
      for (idx = [0 : 10]) {
        C = idx * Cfull / 10;
        /* see separate design document for this equation that maps a "quantity"
           to a distance */
        R = (-Hub + sqrt(Hub^2 + 4*C)) / 2;
        mark_mult = (idx % 5) == 0 ? 2 : 1;
        translate([-(Hub/2 + R), -(scale_slot_width/2 + scalemark_length * mark_mult + 1), -epsilon])
          cube([scalemark_width, scalemark_length * mark_mult, scalemark_depth]);
      }
    }
  }
}

module reel_cover() {
  chamfer = Flange_Thickness / 2;
  wall = 1.2;
  rim = 0.4;
  reelheight = Tape_Width + Tape_Hub_Tolerance + 2*Flange_Thickness;
  coverheight = reelheight + Flange_Thickness + rim;
  inner_diameter = Diameter + 2*Tolerance;
  outer_diameter = inner_diameter + 2*wall;
  
  difference() {
    /* outer shell */
    hull() {
      cylinder(d = outer_diameter - 2*chamfer, h = epsilon);
      translate([0, 0, chamfer])
        cylinder(d = outer_diameter, h = epsilon);
      translate([0, 0, coverheight - chamfer - epsilon])
        cylinder(d = outer_diameter, h = epsilon);
      translate([0, 0, coverheight - epsilon])
        cylinder(d = outer_diameter - 2*chamfer, h = epsilon);
    }
    
    /* cut-outs */
    translate([0, 0, -epsilon])
      cylinder(d = inner_diameter - 2*Flange_Edge_Width, h = coverheight + 2*epsilon);
    translate([0, 0, Flange_Thickness])
      cylinder(d = inner_diameter, h = reelheight);
    translate([0, 0, coverheight - rim])
      cylinder(d = inner_diameter - 2*rim, h = rim + epsilon);
    translate([0, 0, coverheight - rim])
      cylinder(d1 = inner_diameter, d2 = inner_diameter - 2*rim, h = rim/2);
  }
}

gap = 4;
reel_half(TopSide);
translate([Diameter + gap, 0, 0])
  reel_half(BottomSide);
if (Include_Cover)
  translate([Diameter/2 + gap, Diameter*0.87 + gap, 0])
  reel_cover();